@@ -7,9 +7,12 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <errno.h>
 
+#include "insns.h"
 #include "nasm.h"
 #include "nasmlib.h"
 #include "sync.h"
@@ -19,19 +22,21 @@
 
 static const char *help =
 "usage: ndisasm [-a] [-i] [-h] [-r] [-u] [-b bits] [-o origin] [-s sync...]\n"
-"               [-e bytes] [-k start,bytes] file\n"
+"               [-e bytes] [-k start,bytes] [-p vendor] file\n"
 "   -a or -i activates auto (intelligent) sync\n"
 "   -u sets USE32 (32-bit mode)\n"
 "   -b 16 or -b 32 sets number of bits too\n"
 "   -h displays this text\n"
 "   -r displays the version number\n"
 "   -e skips <bytes> bytes of header\n"
-"   -k avoids disassembling <bytes> bytes from position <start>\n";
+"   -k avoids disassembling <bytes> bytes from position <start>\n"
+"   -p selects the preferred vendor instruction set (intel, amd, cyrix, idt)\n";
 
 static void output_ins (unsigned long, unsigned char *, int, char *);
 static void skip (unsigned long dist, FILE *fp);
 
-int main(int argc, char **argv) {
+int main(int argc, char **argv) 
+{
     unsigned char buffer[INSN_MAX * 2], *p, *q;
     char outbuf[256];
     char *pname = *argv;
@@ -40,6 +45,8 @@ int main(int argc, char **argv) {
     int lenread, lendis;
     int autosync = FALSE;
     int bits = 16;
+    int eof = FALSE;
+    unsigned long prefer = 0;
     int rn_error;
     long offset;
     FILE *fp;
@@ -49,7 +56,7 @@ int main(int argc, char **argv) {
 
     while (--argc) {
 	char *v, *vv, *p = *++argv;
-	if (*p == '-') {
+	if (*p == '-' && p[1]) {
 	    p++;
 	    while (*p) switch (tolower(*p)) {
 	      case 'a':		       /* auto or intelligent sync */
@@ -60,11 +67,9 @@ int main(int argc, char **argv) {
 	      case 'h':
 		fprintf(stderr, help);
 		return 0;
-		break;
 	      case 'r':
 		fprintf(stderr, "NDISASM version " NASM_VER "\n");
 		return 0;
-		break;
 	      case 'u':		       /* USE32 */
 		bits = 32;
 		p++;
@@ -155,6 +160,27 @@ int main(int argc, char **argv) {
 		add_sync (nextsync, synclen);
 		p = "";		       /* force to next argument */
 		break;
+	    case 'p':		       /* preferred vendor */
+		v = p[1] ? p+1 : --argc ? *++argv : NULL;
+		if (!v) {
+		    fprintf(stderr, "%s: `-p' requires an argument\n", pname);
+		    return 1;
+		}
+		if ( !strcmp(v, "intel") ) {
+		  prefer = 0;	/* Default */
+		} else if ( !strcmp(v, "amd") ) {
+		  prefer = IF_AMD|IF_3DNOW;
+		} else if ( !strcmp(v, "cyrix") ) {
+		  prefer = IF_CYRIX|IF_3DNOW;
+		} else if ( !strcmp(v, "idt") || !strcmp(v, "centaur") ||
+			    !strcmp(v, "winchip") ) {
+		  prefer = IF_3DNOW;
+		} else {
+		  fprintf(stderr, "%s: unknown vendor `%s' specified with `-p'\n", pname, v);
+		  return 1;
+		}
+		p = "";		       /* force to next argument */
+		break;
 	    }
 	} else if (!filename) {
 	    filename = p;
@@ -169,7 +195,16 @@ int main(int argc, char **argv) {
 	return 0;
     }
 
-    fp = fopen(filename, "rb");
+    if (strcmp(filename, "-")) {
+	fp = fopen(filename, "rb");
+	if (!fp) {
+	    fprintf(stderr, "%s: unable to open `%s': %s\n",
+		    pname, filename, strerror(errno));
+	    return 1;
+	}
+    } else
+	fp = stdin;
+
     if (initskip > 0)
 	skip (initskip, fp);
 
@@ -185,7 +220,12 @@ int main(int argc, char **argv) {
 	unsigned long to_read = buffer+sizeof(buffer)-p;
 	if (to_read > nextsync-offset-(p-q))
 	    to_read = nextsync-offset-(p-q);
-	lenread = fread (p, 1, to_read, fp);
+	if (to_read) {
+	    lenread = fread (p, 1, to_read, fp);
+	    if (lenread == 0)
+		eof = TRUE;	       /* help along systems with bad feof */
+	} else
+	    lenread = 0;
 	p += lenread;
 	if (offset == nextsync) {
 	    if (synclen) {
@@ -197,7 +237,7 @@ int main(int argc, char **argv) {
 	    nextsync = next_sync (offset, &synclen);
 	}
 	while (p > q && (p - q >= INSN_MAX || lenread == 0)) {
-	    lendis = disasm (q, outbuf, bits, offset, autosync);
+	    lendis = disasm (q, outbuf, bits, offset, autosync, prefer);
 	    if (!lendis || lendis > (p - q) ||
 		lendis > nextsync-offset)
 		lendis = eatbyte (q, outbuf);
@@ -213,13 +253,17 @@ int main(int argc, char **argv) {
 	    p -= (q - buffer);
 	    q = buffer;
 	}
-    } while (lenread > 0 || !feof(fp));
-    fclose (fp);
+    } while (lenread > 0 || !(eof || feof(fp)));
+
+    if (fp != stdin)
+	fclose (fp);
+
     return 0;
 }
 
 static void output_ins (unsigned long offset, unsigned char *data,
-			int datalen, char *insn) {
+			int datalen, char *insn) 
+{
     int bytes;
     printf("%08lX  ", offset);
 
@@ -248,7 +292,8 @@ static void output_ins (unsigned long offset, unsigned char *data,
  * Skip a certain amount of data in a file, either by seeking if
  * possible, or if that fails then by reading and discarding.
  */
-static void skip (unsigned long dist, FILE *fp) {
+static void skip (unsigned long dist, FILE *fp) 
+{
     char buffer[256];		       /* should fit on most stacks :-) */
 
     /*
